@@ -30,9 +30,11 @@ from pymongo import common
 from pymongo import periodic_executor
 from pymongo.pool import PoolOptions
 from pymongo.topology_description import (updated_topology_description,
-                                          TOPOLOGY_TYPE,
-                                          TopologyDescription)
+                                          _updated_topology_description_srv_polling,
+                                          TopologyDescription,
+                                          SRV_POLLING_TOPOLOGIES, TOPOLOGY_TYPE)
 from pymongo.errors import ServerSelectionTimeoutError, ConfigurationError
+from pymongo.monitor import SrvMonitor
 from pymongo.monotonic import time as _time
 from pymongo.server import Server
 from pymongo.server_selectors import (any_server_selector,
@@ -72,7 +74,7 @@ class Topology(object):
 
         # Create events queue if there are publishers.
         self._events = None
-        self._events_thread = None
+        self.__events_executor = None
 
         if self._publish_server or self._publish_tp:
             self._events = Queue.Queue(maxsize=100)
@@ -125,10 +127,14 @@ class Topology(object):
             # We strongly reference the executor and it weakly references
             # the queue via this closure. When the topology is freed, stop
             # the executor soon.
-            weak = weakref.ref(self._events)
+            weak = weakref.ref(self._events, executor.close)
             self.__events_executor = executor
             executor.open()
 
+        self._srv_monitor = None
+        if self._settings.fqdn is not None:
+            self._srv_monitor = SrvMonitor(self, self._settings)
+
     def open(self):
         """Start monitoring, or restart after a fork.
 
@@ -190,7 +196,7 @@ class Topology(object):
         now = _time()
         end_time = now + timeout
         server_descriptions = self._description.apply_selector(
-            selector, address)
+            selector, address, custom_selector=self._settings.server_selector)
 
         while not server_descriptions:
             # No suitable servers.
@@ -209,7 +215,8 @@ class Topology(object):
             self._description.check_compatible()
             now = _time()
             server_descriptions = self._description.apply_selector(
-                selector, address)
+                selector, address,
+                custom_selector=self._settings.server_selector)
 
         self._description.check_compatible()
         return server_descriptions
@@ -271,6 +278,14 @@ class Topology(object):
                 self._listeners.publish_topology_description_changed,
                 (td_old, self._description, self._topology_id)))
 
+        # Shutdown SRV polling for unsupported cluster types.
+        # This is only applicable if the old topology was Unknown, and the
+        # new one is something other than Unknown or Sharded.
+        if self._srv_monitor and (td_old.topology_type == TOPOLOGY_TYPE.Unknown
+                                  and self._description.topology_type not in
+                                  SRV_POLLING_TOPOLOGIES):
+            self._srv_monitor.close()
+
         # Wake waiters in select_servers().
         self._condition.notify_all()
 
@@ -290,6 +305,28 @@ class Topology(object):
                     self._description.has_server(server_description.address)):
                 self._process_change(server_description)
 
+    def _process_srv_update(self, seedlist):
+        """Process a new seedlist on an opened topology.
+        Hold the lock when calling this.
+        """
+        td_old = self._description
+        self._description = _updated_topology_description_srv_polling(
+            self._description, seedlist)
+
+        self._update_servers()
+
+        if self._publish_tp:
+            self._events.put((
+                self._listeners.publish_topology_description_changed,
+                (td_old, self._description, self._topology_id)))
+
+    def on_srv_update(self, seedlist):
+        """Process a new list of nodes obtained from scanning SRV records."""
+        # We do no I/O holding the lock.
+        with self._lock:
+            if self._opened:
+                self._process_srv_update(seedlist)
+
     def get_server_by_address(self, address):
         """Get a Server or None.
 
@@ -372,12 +409,18 @@ class Topology(object):
         Do *not* request an immediate check.
         """
         with self._lock:
-            self._reset_server(address)
+            self._reset_server(address, reset_pool=True)
 
     def reset_server_and_request_check(self, address):
         """Clear our pool for a server, mark it Unknown, and check it soon."""
         with self._lock:
-            self._reset_server(address)
+            self._reset_server(address, reset_pool=True)
+            self._request_check(address)
+
+    def mark_server_unknown_and_request_check(self, address):
+        """Mark a server Unknown, and check it soon."""
+        with self._lock:
+            self._reset_server(address, reset_pool=False)
             self._request_check(address)
 
     def update_pool(self):
@@ -394,7 +437,14 @@ class Topology(object):
 
             # Mark all servers Unknown.
             self._description = self._description.reset()
-            self._update_servers()
+            for address, sd in self._description.server_descriptions().items():
+                if address in self._servers:
+                    self._servers[address].description = sd
+
+            # Stop SRV polling thread.
+            if self._srv_monitor:
+                self._srv_monitor.close()
+
             self._opened = False
 
         # Publish only after releasing the lock.
@@ -470,12 +520,17 @@ class Topology(object):
             if self._publish_tp or self._publish_server:
                 self.__events_executor.open()
 
+            # Start the SRV polling thread.
+            if self._srv_monitor and (self.description.topology_type in
+                                      SRV_POLLING_TOPOLOGIES):
+                self._srv_monitor.open()
+
         # Ensure that the monitors are open.
         for server in itervalues(self._servers):
             server.open()
 
-    def _reset_server(self, address):
-        """Clear our pool for a server and mark it Unknown.
+    def _reset_server(self, address, reset_pool):
+        """Mark a server Unknown and optionally reset it's pool.
 
         Hold the lock when calling this. Does *not* request an immediate check.
         """
@@ -483,7 +538,8 @@ class Topology(object):
 
         # "server" is None if another thread removed it from the topology.
         if server:
-            server.reset()
+            if reset_pool:
+                server.reset()
 
             # Mark this server Unknown.
             self._description = self._description.reset_server(address)
@@ -529,7 +585,14 @@ class Topology(object):
                 self._servers[address] = server
                 server.open()
             else:
+                # Cache old is_writable value.
+                was_writable = self._servers[address].description.is_writable
+                # Update server description.
                 self._servers[address].description = sd
+                # Update is_writable value of the pool, if it changed.
+                if was_writable != sd.is_writable:
+                    self._servers[address].pool.update_is_writable(
+                        sd.is_writable)
 
         for address, server in list(self._servers.items()):
             if not self._description.has_server(address):
@@ -551,7 +614,8 @@ class Topology(object):
             ssl_context=options.ssl_context,
             ssl_match_hostname=options.ssl_match_hostname,
             event_listeners=options.event_listeners,
-            appname=options.appname)
+            appname=options.appname,
+            driver=options.driver)
 
         return self._settings.pool_class(address, monitor_pool_options,
                                          handshake=False)
